import React, { Component } from 'react'
import { DndProvider } from 'react-dnd'
import { withRouter } from 'react-router'
import { is, fromJS } from 'immutable'
import HTML5Backend from 'react-dnd-html5-backend'
import { ConfigProvider, notification, Modal, Collapse, Switch, Button, message, Spin, Typography } from 'antd'
import { DoubleLeftOutlined, DoubleRightOutlined, HomeOutlined, LoginOutlined, ArrowLeftOutlined, EyeOutlined, EyeInvisibleOutlined } from '@ant-design/icons'
import moment from 'moment'
import md5 from 'md5'

import Api from '@/api'
import Utils, { setGLOBFuncs } from '@/utils/utils.js'
import antdZhCN from 'antd/es/locale/zh_CN'
import MKEmitter from '@/utils/events.js'
import MenuUtils, { getTables, getFuncsAndInters } from '@/utils/utils-custom.js'
import asyncComponent from '@/utils/asyncComponent'

import '@/assets/css/design.scss'
import './index.scss'

const { Panel } = Collapse
const { confirm } = Modal
const { Paragraph } = Typography

const Header = asyncComponent(() => import('@/mob/header'))
const Debug = asyncComponent(() => import('@/menu/debug'))
const MenuForm = asyncComponent(() => import('./menuform'))
const MobShell = asyncComponent(() => import('@/mob/mobshell'))
const CreateView = asyncComponent(() => import('@/pc/createview'))
const Transfer = asyncComponent(() => import('@/pc/transfer'))
const Versions = asyncComponent(() => import('@/menu/versions'))
const ViewNodes = asyncComponent(() => import('@/menu/viewnodes'))
const SourceWrap = asyncComponent(() => import('@/mob/modulesource'))
const Modulecell = asyncComponent(() => import('@/menu/modulecell'))
const TableNodes = asyncComponent(() => import('@/menu/tablenodes'))
const BgController = asyncComponent(() => import('@/pc/bgcontroller'))
const ReplaceField = asyncComponent(() => import('@/menu/replaceField'))
const NormalCss = asyncComponent(() => import('@/menu/normalCss'))
const SysInterface = asyncComponent(() => import('@/menu/sysinterface'))
const Quotecomponent = asyncComponent(() => import('@/pc/quotecomponent'))
const PasteController = asyncComponent(() => import('@/menu/pastecontroller'))
const StyleController = asyncComponent(() => import('@/menu/stylecontroller'))
const UrlFieldComponent = asyncComponent(() => import('@/menu/urlfieldcomponent'))
const PictureController = asyncComponent(() => import('@/menu/picturecontroller'))
const ModalController = asyncComponent(() => import('@/mob/modalconfig/controller'))
const SearchController = asyncComponent(() => import('@/mob/searchconfig/controller'))
const StyleCombController = asyncComponent(() => import('@/menu/stylecombcontroller'))
const StyleCombControlButton = asyncComponent(() => import('@/menu/stylecombcontrolbutton'))
const TableComponent = asyncComponent(() => import('@/templates/sharecomponent/tablecomponent'))
const PopView = asyncComponent(() => import('./popview'))

sessionStorage.setItem('editMenuType', 'menu') // 编辑菜单类型
sessionStorage.setItem('appType', 'mob')       // 应用类型
document.body.className = ''
window.GLOB.UserComponentMap = new Map() // 缓存用户自定义组件
window.GLOB.TabsMap = new Map()          // 缓存用户操作的标签页
window.GLOB.CacheIndependent = new Map()
window.GLOB.urlFields = []               // url变量
window.GLOB.customMenu = null            // 保存菜单信息

class MobDesign extends Component {
  state = {
    loading: true,
    MenuId: '',
    MenuName: '',
    MenuNo: '',
    activeKey: 'basedata',
    menuloading: false,
    oriConfig: null,
    config: null,
    settingshow: true,
    controlshow: true,
    comloading: false,
    adapters: [],
    viewType: 'menu',
    view: '',
    eyeopen: false,
    needUpdate: false
  }

  UNSAFE_componentWillMount() {
    if (sessionStorage.getItem('devError') === 'true') {
      sessionStorage.clear()
      window.history.replaceState(null, null, window.location.href.split('#')[0] + '#/login')
      window.location.reload()
      return
    }
    
    if (!sessionStorage.getItem('UserID')) {
      sessionStorage.removeItem('editMenuType')
      sessionStorage.removeItem('appType')
      this.props.history.replace('/login')
      return
    }
    
    if (window.GLOB.memberLevel < 30) return

    window.GLOB.developing = true

    try {
      let param = JSON.parse(window.decodeURIComponent(window.atob(this.props.match.params.param)))

      if (param.type === 'app') {
        sessionStorage.setItem('appId', param.ID || '')
        sessionStorage.setItem('appName', param.remark || '')
        sessionStorage.setItem('lang', param.lang || 'zh-CN')
        sessionStorage.setItem('kei_no', param.kei_no || '')
        sessionStorage.setItem('typename', param.typename || 'mob')
        sessionStorage.setItem('adapter', param.adapter || '')
        sessionStorage.setItem('sysBgColor', param.sysBgColor || '#ffffff')
        sessionStorage.setItem('direction', param.direction || 'vertical')
        sessionStorage.setItem('userbind', param.userbind || '')
        sessionStorage.setItem('instantMessage', param.instantMessage || '')

        this.getAppMessage(param.MenuID)
      } else if (param.type === 'view') {
        window.GLOB.winWidth = 420
        window.GLOB.winHeight = 738
        window.GLOB.shellWidth = 376
        window.GLOB.shellHeight = 680
        window.GLOB.curDate = moment().format('YYYY-MM-DD')

        let adapters = sessionStorage.getItem('adapter')
        if (adapters) {
          adapters = adapters.split(',')
        } else {
          adapters = []
        }

        this.setState({
          adapters,
          MenuId: param.MenuID,
          viewType: /^userbind/.test(param.MenuID) ? 'userbind' : 'menu'
        }, () => {
          this.getMenuParam(param)
        })
      }
    } catch (e) {
      notification.warning({
        top: 92,
        message: '菜单信息解析错误！',
        duration: 5
      })
    }
  }

  UNSAFE_componentWillReceiveProps(nextProps) {
    if (this.props.match.params.param !== nextProps.match.params.param) {
      window.location.reload()
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentDidMount () {
    if (!sessionStorage.getItem('UserID')) return
    if (window.GLOB.memberLevel < 30) {
      document.getElementById('mk-mob-design-view').innerHTML = '<div style="text-align: center; font-size: 30px; margin-top: 40vh; height: 100vh; background: #fff;">本应用没有PC端页面的编辑权限，请联系管理员！</div>'
      return
    }
    MKEmitter.addListener('changePopview', this.initPopview)
    MKEmitter.addListener('triggerMenuSave', this.submitConfig)
    MKEmitter.addListener('changeEditMenu', this.changeEditMenu)
    setTimeout(() => {
      this.getRoleFields()
      setGLOBFuncs()
    }, 1000)

    document.onkeydown = (event) => {
      let e = event || window.event
      let keyCode = e.keyCode || e.which || e.charCode
      let preKey = ''

      if (e.ctrlKey) {
        preKey = 'ctrl'
      }
      if (e.shiftKey) {
        preKey = 'shift'
      } else if (e.altKey) {
        preKey = 'alt'
      }
      
      if (!preKey || !keyCode) return
      
      let _shortcut = `${preKey}+${keyCode}`

      if (_shortcut === 'ctrl+83') {
        let modals = document.querySelectorAll('.mk-pop-modal')
        let msg = null
        for (let i = 0; i < modals.length; i++) {
          if (msg) {
            break
          }

          let node = modals[i].querySelector('.mk-com-name')

          if (node) {
            msg = node.innerText
          }
        }
        if (msg) {
          notification.warning({
            top: 92,
            message: '请保存' + msg,
            duration: 5
          })
          return false
        }

        let node = document.getElementById('save-modal-config')
        if (!node) {
          node = document.getElementById('save-config')
        }

        if (node) {
          node.click()
        }
        return false
      }
    }
  }

  /**
   * @description 组件销毁，清除state更新
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('changePopview', this.initPopview)
    MKEmitter.removeListener('triggerMenuSave', this.submitConfig)
    MKEmitter.removeListener('changeEditMenu', this.changeEditMenu)
  }

  initPopview = (card, btn) => {
    const { config } = this.state

    if (!this.checkBase()) {
      return
    }

    let _btn = fromJS(btn).toJS()

    if (_btn.config) {
      _btn.config.uuid = _btn.uuid
      _btn.config.MenuID = _btn.uuid
      _btn.config.ParentId = card.uuid
      _btn.config.MenuName = _btn.label

      _btn.config.components = this.updateComponents(_btn.config.components || [])
    } else {
      _btn.config = {
        uuid: _btn.uuid,
        MenuID: _btn.uuid,
        ParentId: card.uuid,
        enabled: false,
        MenuName: _btn.label,
        tables: config.tables || [],
        Template: 'CustomPage',
        components: [],
        viewType: 'popview',
        style: { backgroundColor: '#ffffff', backgroundImage: '', paddingTop: '16px', paddingBottom: '40px', paddingLeft: '16px', paddingRight: '16px' }
      }
    }

    this.setState({view: 'popview', popConfig: _btn})
  }

  closePop = () => {
    const {config} = this.state

    sessionStorage.setItem('editMenuType', 'menu')

    window.GLOB.urlFields = config.urlFields || []
    window.GLOB.customMenu = config

    this.setState({view: '', popConfig: null})
  }

  submitPopConfig = (btnconfig) => {
    let parents = {[btnconfig.ParentId]: true}
    let popbtns = {[btnconfig.uuid]: fromJS(btnconfig).toJS()}
    let config = fromJS(this.state.config).toJS()

    config.components = this.setPopView(config.components, parents, popbtns)

    this.setState({ config }, () => {
      this.submitConfig()
    })
  }

  setPopView = (components, parents, popbtns) => {
    return components.map(item => {
      if (item.type === 'tabs') {
        item.subtabs.forEach(tab => {
          tab.components = this.setPopView(tab.components, parents, popbtns)
        })
      } else if (item.type === 'group') {
        item.components = this.setPopView(item.components, parents, popbtns)
      } else if (parents[item.uuid]) {
        this.setpopConfig(item, popbtns)
      }
      return item
    })
  }

  setpopConfig = (config, popbtns) => {
    config.subcards && config.subcards.forEach(item => {
      item.elements.forEach(cell => {
        if (cell.eleType !== 'button') return
        if (cell.OpenType === 'popview' && popbtns[cell.uuid]) {
          cell.config = popbtns[cell.uuid]
        }
      })
    })

    if (config.cols) {
      let loopCol = (cols) => {
        cols.forEach(col => {
          if (col.type === 'colspan') {
            loopCol(col.subcols)
          } else if (col.type === 'custom') {
            col.elements.forEach(cell => {
              if (cell.eleType !== 'button') return
              if (cell.OpenType === 'popview' && popbtns[cell.uuid]) {
                cell.config = popbtns[cell.uuid]
              }
            })
          }
        })
      }
      loopCol(config.cols)
    }

    config.elements && config.elements.forEach(cell => {
      if (cell.eleType !== 'button') return
      if (cell.OpenType === 'popview' && popbtns[cell.uuid]) {
        cell.config = popbtns[cell.uuid]
      }
    })
  
    config.action && config.action.forEach(cell => {
      if (cell.OpenType === 'popview' && popbtns[cell.uuid]) {
        cell.config = popbtns[cell.uuid]
      }
    })

    config.$tables = getTables(config)
  }

  checkBase = () => {
    const { config } = this.state

    if (!config.MenuName || !config.MenuNo) {
      notification.warning({
        top: 92,
        message: '请完善菜单基本信息！',
        duration: 5
      })
      return false
    }
    return true
  }

  changeEditMenu = (menu) => {
    const { oriConfig, config } = this.state

    if (!is(fromJS(oriConfig || {}), fromJS(config || {}))) {
      notification.warning({
        top: 92,
        message: '配置信息未保存！',
        duration: 5
      })
      return
    } else if (['AIService'].includes(menu.MenuID)) {
      notification.warning({
        top: 92,
        message: '系统页面不可编辑！',
        duration: 5
      })
      return
    } else if (menu.routerUrl) {
      this.props.history.push(menu.routerUrl)
      return
    }

    let param = {
      MenuID: menu.MenuID,
      copyMenuId: menu.copyMenuId || '',
      clearMenu: menu.clearMenu !== 'false',
      type: 'view'
    }

    param.MenuNo = menu.MenuNo || ''
    param.MenuName = menu.MenuName || ''

    param = window.btoa(window.encodeURIComponent(JSON.stringify(param)))

    if (param === this.props.match.params.param) return

    this.props.history.push('/mobdesign/' + param)
  }

  getAppMessage = (MenuID) => {
    Api.getCloudConfig({
      func: 's_get_keyids',
      bid: sessionStorage.getItem('appId')
    }).then(res => {
      if (!res.status) {
        notification.warning({
          top: 92,
          message: res.message,
          duration: 5
        })
        return
      }

      let homeId = ''
      let appViewList = []
      if (res.data && res.data.length > 0) {
        appViewList = res.data
        appViewList.forEach(item => {
          if (item.keys_type === 'index') {
            homeId = item.keys_id
          }
        })
      }

      if (!homeId) {
        homeId = MenuID || Utils.getuuid()

        let param = {
          func: 's_kei_link_keyids_addupt',
          BID: sessionStorage.getItem('appId'),
          exec_type: 'x',
          LText: ''
        }

        appViewList.unshift({
          appkey: window.GLOB.appkey || '',
          bid: sessionStorage.getItem('appId') || '',
          kei_no: sessionStorage.getItem('kei_no') || '',
          keys_id: homeId,
          keys_type: 'index',
          remark: '首页'
        })

        param.LText = appViewList.map(item => `select '${item.keys_id}','${item.keys_type}','${item.kei_no}','${item.appkey}','${item.bid}','${sessionStorage.getItem('CloudUserID')}','${item.remark}'`)
        param.LText = param.LText.join(' union all ')
        param.LText = Utils.formatOptions(param.LText, 'x')
  
        param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
        param.secretkey = Utils.encrypt('', param.timestamp)

        Api.getCloudConfig(param).then(result => {
          if (!result.status) {
            notification.warning({
              top: 92,
              message: result.message,
              duration: 5
            })
          } else {
            sessionStorage.setItem('appViewList', JSON.stringify(appViewList))
            this.props.history.replace('/mobdesign/' + window.btoa(window.encodeURIComponent(JSON.stringify({MenuID: homeId, type: 'view'}))))
          }
        })
      } else {
        sessionStorage.setItem('appViewList', JSON.stringify(appViewList))
        this.props.history.replace('/mobdesign/' + window.btoa(window.encodeURIComponent(JSON.stringify({MenuID: MenuID || homeId, type: 'view'}))))
      }
    })
  }

  closeView = () => {
    const { oriConfig, config } = this.state

    if (!config) {
      window.close()
    } else if (!is(fromJS(oriConfig || {}), fromJS(config || {}))) {
      confirm({
        title: '配置信息未保存，确定关闭吗？',
        content: '',
        onOk() {
          window.close()
        },
        onCancel() {}
      })
    } else {
      window.close()
    }
  }

  backView = () => {
    const { oriConfig, config } = this.state

    if (!config) {
      window.history.back()
    } else if (!is(fromJS(oriConfig || {}), fromJS(config || {}))) {
      confirm({
        title: '配置信息未保存，确定后退吗？',
        content: '',
        onOk() {
          window.history.back()
        },
        onCancel() {}
      })
    } else {
      window.history.back()
    }
  }

  getMenuParam = (urlParam) => {
    const { MenuId } = this.state

    let param = {
      func: 'sPC_Get_LongParam',
      TypeCharOne: sessionStorage.getItem('kei_no'),
      typename: sessionStorage.getItem('typename'),
      MenuID: MenuId
    }

    Api.getCloudConfig(param).then(result => {
      if (!result.status) {
        notification.warning({
          top: 92,
          message: result.message,
          duration: 5
        })
        this.setState({loading: false})
        return
      } else if (!result.LongParam && urlParam.copyMenuId) {
        this.getCopyParam(urlParam)
      } else {
        let config = null

        try {
          config = result.LongParam ? JSON.parse(window.decodeURIComponent(window.atob(result.LongParam))) : null
        } catch (e) {
          console.warn('Parse Failure')
          config = null
        }

        if (/^userbind/.test(MenuId)) {
          this.setUserBindMenu(config, result)
          return
        }

        if (!config) {
          config = {
            version: 2.0,
            uuid: MenuId,
            MenuID: MenuId,
            Template: 'webPage',
            enabled: false,
            MenuName: urlParam.MenuName || '',
            MenuNo: '',
            tables: [],
            components: [],
            viewType: 'menu',
            statusBarbgColor: sessionStorage.getItem('sysBgColor') || '#ffffff',
            style: {
              backgroundColor: sessionStorage.getItem('sysBgColor') || '#ffffff', backgroundImage: ''
            }
          }
        }
        
        config.uuid = MenuId
        config.MenuID = MenuId
        config.Template = 'webPage'
        config.open_edition = result.open_edition || ''
        config.direction = config.direction || sessionStorage.getItem('direction') || 'vertical'
        window.GLOB.urlFields = config.urlFields || []

        if (sessionStorage.getItem('typename') === 'pad') {
          if (config.direction !== 'vertical') {
            window.GLOB.winWidth = 992
            window.GLOB.winHeight = 690
            window.GLOB.shellWidth = 853
            window.GLOB.shellHeight = 640
          } else {
            window.GLOB.winWidth = 736
            window.GLOB.winHeight = 945
            window.GLOB.shellWidth = 640
            window.GLOB.shellHeight = 853
          }
        }

        if (config.version !== 2.0) {
          config.components = this.collectTB(config.components)
          config.version = 2.0

          this.setState({
            needUpdate: true
          })
        } else {
          config.components = this.updateComponents(config.components)
        }

        let navItem = null
        config.components.forEach(item => {
          if (item.type === 'navbar') {
            navItem = fromJS(item).toJS()
          }
        })

        if (!navItem) {
          this.setState({
            oriConfig: fromJS(config).toJS(),
            config: config,
            loading: false
          })
          window.GLOB.customMenu = config
        } else {
          this.jointComponents(config, navItem)
        }
      }
    })
    this.getAppMenus()
  }

  setUserBindMenu = (config, result) => {
    const { MenuId } = this.state

    if (!config) {
      config = {
        version: 2.0,
        uuid: MenuId,
        MenuID: MenuId,
        Template: 'webPage',
        enabled: false,
        MenuName: '用户绑定',
        MenuNo: 'user_bind',
        tables: [],
        components: [],
        viewType: 'userbind',
        statusBarbgColor: '#ffffff',
        style: {
          backgroundColor: '#ffffff', backgroundImage: ''
        }
      }

      config.components.push({
        subcards:[
          {
            uuid: Utils.getuuid(), setting: {width:24, primaryId: '', click: ''}, style: {}, backStyle: {},
            elements: [{maxWidth: 120, datatype: 'static', width: 24, marks: null, url: 'http://cloud.mk9h.cn/Content/images/20220120/cb5cd13a-2ef4-41cc-ab6a-a202f1c04da5.png', style: {borderRadius: '50%'}, eleType: 'picture', link: '', uuid: Utils.getuuid(), lenWidRadio: '1:1', $type: ''}],
            backElements: []
          }
        ],
        headerStyle: {}, width: 24, scripts: [], pageable: false,
        wrap: {name: 'logo', width: 24, datatype: 'static', cardType: '', blacklist: []},
        name: 'logo', floor: 1, switchable: true,
        setting:{interType: 'system'}, style:{paddingTop: '8vh', paddingBottom: '10vh'}, format: 'object', subtype: 'propcard', type: 'card', uuid: Utils.getuuid(), columns: []
      })

      config.components.push({
        uuid: Utils.getuuid(),
        type: 'login',
        floor: 1,
        width: 24,
        name: '用户绑定',
        subtype: 'bindlogin',
        wrap: { name: '用户绑定', width: 24, loginWays: ['uname_pwd'], shortcut: 'none', height: '45vh' },
        style: {},
        loginWays: [
          {type: 'uname_pwd', label: '账号登录', shortcut: 'none'},
          {type: 'sms_vcode', label: '短信登录'},
        ]
      })

      config.components.push({
        subcards:[
          {
            uuid: Utils.getuuid(), setting: {width:24, primaryId: '', click: ''}, style: {}, backStyle: {},
            elements: [{datatype: 'static', width: 24, marks: null, height: null, value: 'Power by Minkesoft', style: { fontSize: '13px', textAlign: 'center'}, eleType: 'text', link: '', uuid: Utils.getuuid(), $type: ''}],
            backElements: []
          }
        ],
        headerStyle: {}, width: 24, scripts: [], pageable: false,
        wrap: {name: 'Power', width: 24, datatype: 'static', cardType: '', blacklist: []},
        name: 'Power', floor: 1, switchable: true,
        setting:{interType: 'system'}, style:{}, format: 'object', subtype: 'propcard', type: 'card', uuid: Utils.getuuid(), columns: []
      })
    }
    
    config.uuid = MenuId
    config.MenuID = MenuId
    config.Template = 'webPage'
    config.open_edition = result.open_edition || ''

    this.setState({
      oriConfig: fromJS(config).toJS(),
      config: config,
      loading: false
    })
    window.GLOB.customMenu = config
  }

  getAppMenus = () => {
    let _param = {
      func: 's_get_app_menus',
      TypeCharOne: sessionStorage.getItem('kei_no'),
      typename: sessionStorage.getItem('typename'),
      LText: `select '${window.GLOB.appkey}'`,
      timestamp: moment().format('YYYY-MM-DD HH:mm:ss')
    }

    _param.secretkey = Utils.encrypt(_param.LText, _param.timestamp)

    Api.getCloudConfig(_param).then(res => {
      if (!res.status) {
        notification.warning({
          top: 92,
          message: res.message,
          duration: 5
        })
        return
      }

      let appIndeList = sessionStorage.getItem('appViewList')
      appIndeList = JSON.parse(appIndeList)
      appIndeList = appIndeList.map(item => (item.keys_type === 'navbar' ? item.keys_id : '')).join(',')

      // if (sessionStorage.getItem('userbind')) {
      //   appIndeList = appIndeList + ',' + sessionStorage.getItem('userbind')
      // }
      if (sessionStorage.getItem('instantMessage')) {
        appIndeList = appIndeList + ',' + sessionStorage.getItem('instantMessage')
      }

      let menus = res.menus.filter(item => appIndeList.indexOf(item.MenuID) === -1)
      menus = menus.map(item => {
        item.value = item.MenuID
        if (item.MenuNo) {
          item.label = item.MenuName + '（' + item.MenuNo + '）'
        } else {
          item.label = item.MenuName
        }
        return item
      })
      sessionStorage.setItem('appMenus', JSON.stringify(menus))
      sessionStorage.setItem('allMenus', JSON.stringify(res.menus || []))
    })
  }

  getCopyParam = (urlParam) => {
    const { MenuId } = this.state

    let param = {
      func: 'sPC_Get_LongParam',
      TypeCharOne: sessionStorage.getItem('kei_no'),
      typename: sessionStorage.getItem('typename'),
      MenuID: urlParam.copyMenuId
    }

    Api.getCloudConfig(param).then(result => {
      if (!result.status) {
        notification.warning({
          top: 92,
          message: result.message,
          duration: 5
        })
        this.setState({loading: false})
        return
      } else if (!result.LongParam) {
        notification.warning({
          top: 92,
          message: '未查询到复制菜单配置信息！',
          duration: 5
        })
      }

      let config = null

      try {
        config = result.LongParam ? JSON.parse(window.decodeURIComponent(window.atob(result.LongParam))) : null
      } catch (e) {
        console.warn('Parse Failure')
        config = null
      }

      
      if (!config) {
        config = {
          version: 2.0,
          uuid: MenuId,
          MenuID: MenuId,
          Template: 'webPage',
          enabled: false,
          MenuName: '',
          MenuNo: '',
          tables: [],
          components: [],
          viewType: 'menu',
          style: {
            backgroundColor: '#ffffff', backgroundImage: '', paddingLeft: '20px', paddingRight: '20px'
          }
        }
      } else {
        let commonId = Utils.getuuid()
        if (config.interfaces && config.interfaces.length > 0) {
          config.interfaces = config.interfaces.map(inter => {
            inter.uuid = md5(commonId + inter.uuid)
            return inter
          })
        }

        config.components = MenuUtils.resetConfig(config.components, commonId, urlParam.clearMenu)

        if (config.version !== 2.0) {
          config.components = this.collectTB(config.components)
          config.version = 2.0

          this.setState({
            needUpdate: true
          })
        } else {
          config.components = this.updateComponents(config.components)
        }

        config.enabled = false
        message.success('复制成功，保存后生效。')
      }
      
      config.uuid = MenuId
      config.MenuID = MenuId
      config.open_edition = ''
      config.Template = 'webPage'
      config.MenuName = urlParam.MenuName || ''
      // config.MenuNo = urlParam.MenuNo || ''
      config.MenuNo = ''
      config.direction = config.direction || sessionStorage.getItem('direction') || 'vertical'

      if (sessionStorage.getItem('typename') === 'pad') {
        if (config.direction !== 'vertical') {
          window.GLOB.winWidth = 992
          window.GLOB.winHeight = 690
          window.GLOB.shellWidth = 853
          window.GLOB.shellHeight = 640
        } else {
          window.GLOB.winWidth = 736
          window.GLOB.winHeight = 945
          window.GLOB.shellWidth = 640
          window.GLOB.shellHeight = 853
        }
      }

      let navItem = null
      config.components.forEach(item => {
        if (item.type === 'navbar') {
          navItem = fromJS(item).toJS()
        }
      })

      if (!navItem) {
        this.setState({
          oriConfig: null,
          config: fromJS(config).toJS(),
          loading: false
        })

        window.GLOB.customMenu = config
      } else {
        this.jointComponents(config, navItem)
      }
    })
  }

  jointComponents = (config, navItem) => {
    Api.getCloudConfig({
      func: 'sPC_Get_LongParam',
      TypeCharOne: sessionStorage.getItem('kei_no'),
      typename: sessionStorage.getItem('typename'),
      MenuID: navItem.uuid
    }).then(res => {
      if (!res.status) {
        notification.warning({
          top: 92,
          message: res.message,
          duration: 5
        })

        config.components = config.components.filter(item => item.type !== 'navbar')
      } else {
        let _config = null
        try {
          _config = res.LongParam ? JSON.parse(window.decodeURIComponent(window.atob(res.LongParam))) : null
        } catch (e) {
          console.warn('Parse Failure')
          _config = null
        }

        if (_config) {
          _config.open_edition = res.open_edition || ''
          config.components = config.components.map(item => {
            if (item.type === 'navbar') {
              return _config
            }
            return item
          })

          window.GLOB.CacheIndependent.set(navItem.uuid, fromJS(_config).toJS())
        } else {
          config.components = config.components.filter(item => item.type !== 'navbar')
        }
      }
      
      this.setState({
        oriConfig: fromJS(config).toJS(),
        config: config,
        loading: false
      })

      window.GLOB.customMenu = config
    })
  }

  updateComponents = (components) => { // 兼容性升级 table
    return components.map(item => {
      if (item.type === 'tabs') {
        item.subtabs.forEach(tab => {
          tab.components = this.updateComponents(tab.components)
        })
      } else if (item.type === 'group') {
        item.components = this.updateComponents(item.components)
      } else if (item.type === 'table') {
        item.cols = item.cols.map(col => {
          if (col.type === 'action') {
            col.type = 'custom'
          }
          return col
        })
      }

      return item
    })
  }

  collectTB = (components) => {
    return components.map(item => {
      if (item.type === 'tabs') {
        item.subtabs.forEach(tab => {
          delete tab.floor
          delete tab.hasSearch
          delete tab.parentId

          tab.components = this.collectTB(tab.components)
        })
      } else if (item.type === 'group') {
        item.components = this.collectTB(item.components)
      } else if (!['search', 'navbar', 'login', 'topbar', 'officialAccount'].includes(item.type)) {
        item.$tables = getTables(item)
      }

      if (item.subtype === 'tablecard') { // 兼容
        item.type = 'card'
      } else if (item.type === 'table') {
        item.cols = item.cols.map(col => {
          if (col.type === 'action') {
            col.type = 'custom'
          }
          return col
        })
      }

      delete item.tabId
      delete item.parentId
      delete item.btnlog
      delete item.floor
      delete item.dataName

      return item
    })
  }

  getMenuMessage = (tbs) => {
    const { config } = this.state
    
    let traversal = (components) => {
      let list = []
      components.forEach(item => {
        if (item.$tables) {
          tbs.push(...item.$tables)
        }

        if (item.plot && item.plot.permission !== 'true') {
          return
        } else if (['login', 'navbar', 'topbar'].includes(item.type) || item.subtype === 'commonbar') {
          return
        } else if (!['tabs', 'group'].includes(item.type) && item.wrap && item.wrap.permission !== 'true') {
          return
        }

        let m = {
          key: item.uuid,
          title: item.name,
          children: []
        }

        if (item.type === 'tabs') {
          let tabs = []
          let mm = []
          item.subtabs.forEach(tab => {
            let s = traversal(tab.components)

            if (s.length === 0) return
            
            tabs.push({
              key: tab.uuid,
              title: tab.label,
              children: s
            })

            mm.push(...s)
          })

          if (tabs.length > 0) {
            m.children = tabs
          }

          if (item.setting.permission === 'true') {
            list.push(m)
          } else if (mm.length) {
            list.push(...mm)
          }
          
          return
        } else if (item.type === 'group') {
          m.children = traversal(item.components)

          if (item.setting.permission === 'true') {
            list.push(m)
          } else if (m.children.length) {
            list.push(...m.children)
          }
          
          return
        } else if (item.type === 'card' || item.type === 'carousel' || item.type === 'timeline') {
          item.action && item.action.forEach(btn => {
            if (btn.hidden === 'true' || btn.permission === 'false') return

            m.children.push({
              key: btn.uuid,
              title: btn.label,
            })
          })
          item.subcards.forEach(card => {
            card.elements && card.elements.forEach(cell => {
              if (cell.eleType !== 'button' || cell.hidden === 'true' || cell.permission === 'false') return

              m.children.push({
                key: cell.uuid,
                title: cell.label,
              })
            })

            if (item.subtype !== 'dualdatacard') return
            
            card.backElements && card.backElements.forEach(cell => {
              if (cell.eleType !== 'button' || cell.hidden === 'true' || cell.permission === 'false') return

              m.children.push({
                key: cell.uuid,
                title: cell.label,
              })
            })
          })
        } else if (item.type === 'balcony') {
          item.elements && item.elements.forEach(cell => {
            if (cell.eleType !== 'button' || cell.hidden === 'true' || cell.permission === 'false') return

            m.children.push({
              key: cell.uuid,
              title: cell.label,
            })
          })
        } else if (item.type === 'menubar') {
          if (item.wrap.title) {
            m.title = item.wrap.title
          }

          m.children = item.subMenus.map(menu => {
            return {
              key: menu.uuid,
              title: menu.setting.name
            }
          })
        } else if (item.type === 'form') {
          if (item.subtype !== 'simpleform') {
            m.children = item.subcards.map(m => {
              return {
                key: m.uuid,
                title: m.setting.title
              }
            })
          }
        } else if (item.type === 'table') {
          item.action && item.action.forEach(btn => {
            if (btn.hidden === 'true' || btn.permission === 'false') return

            m.children.push({
              key: btn.uuid,
              title: btn.label,
            })
          })
          let loopCol = (cols) => {
            cols.forEach(col => {
              if (col.type === 'colspan') {
                loopCol(col.subcols)
              } else if (col.type === 'custom') {
                col.elements.forEach(cell => {
                  if (cell.eleType !== 'button' || cell.hidden === 'true' || cell.permission === 'false') return
                  m.children.push({
                    key: cell.uuid,
                    title: cell.label,
                  })
                })
              }
            })
          }
          loopCol(item.cols)
        }

        list.push(m)
      })

      return list
    }

    let trees = traversal(config.components)

    if (config.interfaces) {
      config.interfaces.forEach(item => {
        if (item.$tables) {
          tbs.push(...item.$tables)
        }
      })
    }

    return trees
  }

  getSubMenus = () => {
    const { config } = this.state
    let menus = []
    let menuObj = {}
    let allMenus = JSON.parse(sessionStorage.getItem('allMenus'))

    allMenus.forEach(item => {
      menuObj[item.MenuID] = item
    })

    let traversal = (components) => {
      components.forEach(item => {
        if (item.type === 'topbar') {
          if (item.wrap.linkmenu && menuObj[item.wrap.linkmenu]) {
            menus.push(menuObj[item.wrap.linkmenu])
          }
          if (item.wrap.menus) {
            item.wrap.menus.forEach(m => {
              if (!menuObj[m.menu]) return

              menus.push(menuObj[m.menu])
            })
          }
        } else if (item.type === 'login') {
          if (item.wrap.linkmenu && menuObj[item.wrap.linkmenu]) {
            menus.push(menuObj[item.wrap.linkmenu])
          }
        } else if (item.type === 'menubar' && item.subtype !== 'commonbar') {
          item.subMenus.forEach(m => {
            if (m.setting.type === 'menu') {
              if (menuObj[m.uuid]) {
                menus.push(menuObj[m.uuid])
              } else {
                menus.push({
                  MenuID: m.uuid,
                  MenuName: m.setting.name,
                  MenuNo: ''
                })
              }
            } else if (m.setting.type === 'linkmenu' && menuObj[m.setting.linkMenuId]) {
              menus.push(menuObj[m.setting.linkMenuId])
            }
          })
        } else if (item.type === 'tabs') {
          item.subtabs.forEach(tab => {
            traversal(tab.components)
          })
        } else if (item.type === 'group') {
          traversal(item.components)
        } else if (item.type === 'card' || item.type === 'carousel' || item.type === 'timeline') {
          item.action && item.action.forEach(btn => {
            if (btn.linkmenu && menuObj[btn.linkmenu]) {
              menus.push(menuObj[btn.linkmenu])
            } else if (btn.openmenu && menuObj[btn.openmenu]) {
              menus.push(menuObj[btn.openmenu])
            }
          })
          item.subcards.forEach(card => {
            if (card.setting.click === 'menu' && menuObj[card.setting.menu]) {
              menus.push(menuObj[card.setting.menu])
            } else if (card.setting.click === 'menus' && card.menus) {
              card.menus.forEach(m => {
                if (menuObj[m.menu]) {
                  menus.push(menuObj[m.menu])
                }
              })
            }
            card.elements && card.elements.forEach(cell => {
              if (cell.linkmenu && menuObj[cell.linkmenu]) {
                menus.push(menuObj[cell.linkmenu])
              } else if (cell.openmenu && menuObj[cell.openmenu]) {
                menus.push(menuObj[cell.openmenu])
              }
            })

            if (item.subtype === 'dualdatacard') {
              card.backElements && card.backElements.forEach(cell => {
                if (cell.linkmenu && menuObj[cell.linkmenu]) {
                  menus.push(menuObj[cell.linkmenu])
                } else if (cell.openmenu && menuObj[cell.openmenu]) {
                  menus.push(menuObj[cell.openmenu])
                }
              })
            }
          })
        } else if (item.type === 'balcony') {
          item.elements && item.elements.forEach(cell => {
            if (cell.linkmenu && menuObj[cell.linkmenu]) {
                menus.push(menuObj[cell.linkmenu])
              } else if (cell.openmenu && menuObj[cell.openmenu]) {
                menus.push(menuObj[cell.openmenu])
              }
          })
        } else if (item.type === 'form') {
          item.subcards.forEach(m => {
            if (m.subButton && m.subButton.linkmenu && menuObj[m.subButton.linkmenu]) {
              menus.push(menuObj[m.subButton.linkmenu])
            }
          })
        } else if (item.type === 'table') {
          item.action && item.action.forEach(btn => {
            if (btn.linkmenu && menuObj[btn.linkmenu]) {
              menus.push(menuObj[btn.linkmenu])
            } else if (btn.openmenu && menuObj[btn.openmenu]) {
              menus.push(menuObj[btn.openmenu])
            }
          })
          let loopCol = (cols) => {
            cols.forEach(col => {
              if (col.type === 'colspan') {
                loopCol(col.subcols)
              } else if (col.type === 'custom') {
                col.elements.forEach(cell => {
                  if (cell.linkmenu && menuObj[cell.linkmenu]) {
                    menus.push(menuObj[cell.linkmenu])
                  } else if (cell.openmenu && menuObj[cell.openmenu]) {
                    menus.push(menuObj[cell.openmenu])
                  }
                })
              }
            })
          }
          loopCol(item.cols)
        }
      })
    }

    traversal(config.components)

    let map = new Map()
    menus = menus.filter(m => {
      if (map.has(m.MenuID)) return false
      map.set(m.MenuID, true)
      return true
    })

    return menus
  }

  getMiniStyle = (config) => {
    config.miniStyle = this.transferStyle(config.style)

    let traversal = (components) => {
      return components.map(item => {
        if (item.style) {
          item.miniStyle = this.transferStyle(item.style)
        }
        if (item.headerStyle) {
          item.miniHeaderStyle = this.transferStyle(item.headerStyle)
        }

        if (item.action && item.action.length > 0) {
          item.action = item.action.map(btn => {
            btn.miniStyle = this.transferStyle(btn.style)
            return btn
          })
        }

        if (item.type === 'tabs') {
          item.subtabs.forEach(tab => {
            tab.components = traversal(tab.components)
          })
        } else if (item.type === 'group') {
          item.components = traversal(item.components)
        } else if (['card', 'carousel', 'timeline'].includes(item.type)) {
          item.subcards.forEach(card => {
            card.miniStyle = this.transferStyle(card.style)
            card.elements = card.elements.map(cell => {
              cell.miniStyle = this.transferStyle(cell.style)
              return cell
            })
          })
        } else if (item.type === 'balcony') {
          item.elements && item.elements.forEach(cell => {
            cell.miniStyle = this.transferStyle(cell.style)
          })
        } else if (item.type === 'menubar') {
          item.subMenus = item.subMenus.map(menu => {
            menu.miniStyle = this.transferStyle(menu.style)
            return menu
          })
        } else if (item.type === 'table') {
          let getCols = (cols) => {
            return cols.map(col => {
              if (col.type === 'colspan') {
                col.subcols = getCols(col.subcols || [])
              } else if (col.type === 'custom') {
                col.elements = col.elements.map(cell => {
                  cell.miniStyle = this.transferStyle(cell.style)
                  return cell
                })
              }
  
              return col
            })
          }
          
          item.cols = getCols(item.cols)
        }

        return item
      })
    }

    config.components = traversal(config.components)

    return config
  }

  transferStyle = (style = {}) => {
    let _style = ''
    Object.keys(style).forEach(key => {
      if (['hShadow', 'vShadow', 'shadowBlur', 'shadowColor'].includes(key)) return
      if (!style[key]) return
      _style += `${key.replace(/[A-Z]/g, m => '-' + m.toLowerCase())}:${style[key]};`
    })

    return _style
  }

  resetSyncQuery = (components) => {
    return components.map(item => {
      if (item.type === 'tabs') {
        item.subtabs.forEach(tab => {
          tab.components = this.resetSyncQuery(tab.components)
        })
      } else if (item.type === 'group') {
        item.components = this.resetSyncQuery(item.components)
      } else if (item.setting && item.setting.interType === 'system' && item.setting.sync === 'true') {
        let sql = ''
        if (item.setting.execute !== 'false' && item.setting.dataresource) {
          sql = item.setting.dataresource
        }

        item.scripts && item.scripts.forEach(script => {
          if (script.status === 'false') return

          sql += script.sql
        })
       
        if (sql.length > 8000) {
          item.setting.sync = 'false'
        }
      }

      return item
    })
  }

  submitConfig = () => {
    const { adapters } = this.state
    let config = fromJS(this.state.config).toJS()

    if (!config.MenuName || !config.MenuNo || (config.cacheUseful === 'true' && !config.cacheTime)) {
      notification.warning({
        top: 92,
        message: '请完善菜单基本信息！',
        duration: 5
      })
      this.setState({
        settingshow: true,
        activeKey: 'basedata'
      })
      return
    } else if (this.checklog()) {
      notification.success({
        top: 92,
        message: '当前配置未修改，无需保存。',
        duration: 5
      })
      MKEmitter.emit('completeSave')
      return
    }

    this.setState({
      menuloading: true
    })

    setTimeout(() => {
      let _pass = this.verifyConfig(config)

      if (config.enabled && !_pass) {
        config.enabled = false
        config.force = true
      } else if (!config.enabled && config.force && _pass) {
        config.enabled = true
        delete config.force
      }

      if (config.cacheUseful !== 'true') {
        config.components = this.resetSyncQuery(config.components)
      }

      let tbs = []
      let roleParam = {type: 'view', version: '1.0', key: config.uuid, title: config.MenuName, children: []}
      roleParam.children = this.getMenuMessage(tbs)
      config.loginview = false
      config.tabview = false

      let arr = []
      tbs = tbs.filter(tb => {
        let _tb = tb.toLowerCase()

        if (arr.includes(_tb)) return false
        arr.push(_tb)

        return true
      })
      tbs.sort()
      if (tbs.length && sessionStorage.getItem('mk_tb_names')) {
        let names = sessionStorage.getItem('mk_tb_names')
        tbs = tbs.filter(tb => names.indexOf(',' + tb.toLowerCase() + ',') > -1)
      }
      tbs = tbs.map(tb => `'${tb}'`).join(';')

      let key = md5(config.uuid + sessionStorage.getItem('kei_no') + sessionStorage.getItem('typename') + sessionStorage.getItem('lang') + tbs.toLowerCase())
      let url = ''

      if (config.tbkey === key) {
        key = ''
      } else {
        let urlparam = {
          type: 'app',
          MenuID: config.uuid,
          ID: sessionStorage.getItem('appId') || '',
          remark: sessionStorage.getItem('appName') || '',
          lang: sessionStorage.getItem('lang') || '',
          kei_no: sessionStorage.getItem('kei_no') || '',
          typename: sessionStorage.getItem('typename') || '',
          adapter: sessionStorage.getItem('adapter') || '',
          sysBgColor: sessionStorage.getItem('sysBgColor') || '',
          MenuName: config.MenuName || '',
        }

        url = window.btoa(window.encodeURIComponent(JSON.stringify(urlparam)))
        config.tbkey = key
      }

      if (roleParam.children.length === 0) {
        roleParam.pass = true
      }

      config.components.forEach(item => {
        if (item.type === 'login') {
          roleParam.login = true
          roleParam.children = []
          config.loginview = true
        } else if (item.type === 'navbar') {
          config.tabview = true
        }
      })

      if (adapters.includes('wxmini')) {
        config = this.getMiniStyle(config)

        if (config.statusBarbgColor && !config.statusBarHexColor && /^rgba/ig.test(config.statusBarbgColor)) {
          let hexify = (color) => {
            let values = color.replace(/rgba?\(/, '').replace(/\)/, '').replace(/[\s+]/g, '').split(',')
            let a = parseFloat(values[3] || 1)
            let r = Math.floor(a * parseInt(values[0]) + (1 - a) * 255)
            let g = Math.floor(a * parseInt(values[1]) + (1 - a) * 255)
            let b = Math.floor(a * parseInt(values[2]) + (1 - a) * 255)
            
            return '#' + ('0' + r.toString(16)).slice(-2) + ('0' + g.toString(16)).slice(-2) + ('0' + b.toString(16)).slice(-2)
          }
          config.statusBarHexColor = hexify(config.statusBarbgColor)
        }
      }

      let subMenus = this.getSubMenus()
      let menus_used_list = subMenus.map(m => `'${config.uuid}','${config.MenuName || ''}','${config.MenuNo || ''}','${m.MenuID}','${m.MenuName}'`).join(';')
      menus_used_list = window.btoa(window.encodeURIComponent(menus_used_list || 'del'))

      let interfaces = getFuncsAndInters(config)
      roleParam.interfaces = interfaces

      let param = {
        func: 'sPC_TrdMenu_AddUpt',
        FstID: 'mk_app',
        SndID: 'mk_app',
        ParentID: 'mk_app',
        MenuID: config.uuid,
        MenuNo: config.MenuNo || '',
        EasyCode: '',
        Template: 'webPage',
        TypeCharOne: sessionStorage.getItem('kei_no'),
        Typename: sessionStorage.getItem('typename'),
        MenuName: config.MenuName || '',
        PageParam: JSON.stringify({Template: 'webPage', interfaces}),
        open_edition: config.open_edition,
        menus_rolelist: window.btoa(window.encodeURIComponent(JSON.stringify(roleParam))),
        LText: '',
        LTexttb: '',
        menus_used_list,
        debug_md5: key,
        debug_url: url,
        debug_list: window.btoa(tbs)
      }

      param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
      param.secretkey = Utils.encrypt('', param.timestamp)

      let _config = fromJS(config).toJS()
      let NavBar = null
      _config.components = _config.components.map(item => {
        if (item.type === 'navbar') {
          NavBar = fromJS(item).toJS()
          return {
            type: 'navbar',
            uuid: item.uuid
          }
        }
        return item
      })

      param.LongParam = window.btoa(window.encodeURIComponent(JSON.stringify(_config)))

      new Promise(resolve => {
        if (!NavBar) {
          resolve({status: true})
        } else {
          let _item = window.GLOB.CacheIndependent.get(NavBar.uuid) || {}
          NavBar.open_edition = _item.open_edition || ''
          if (is(fromJS(_item), fromJS(NavBar))) {
            resolve({status: true})
          } else {
            let roles = {
              type: 'navbar',
              version: '1.0',
              key: NavBar.uuid,
              title: NavBar.name,
              children: []
            }
    
            if (NavBar.wrap.permission === 'true') {
              roles.children = NavBar.menus.map(menu => {
                return {
                  key: menu.MenuID,
                  title: menu.name
                }
              })
            } else {
              roles.pass = true
            }

            let _param = {
              func: 'sPC_TrdMenu_AddUpt',
              FstID: 'mk_app',
              SndID: 'mk_app',
              ParentID: 'mk_app',
              MenuID: NavBar.uuid,
              MenuNo: NavBar.wrap.MenuNo || Utils.getuuid(),
              EasyCode: '',
              Template: NavBar.type,
              TypeCharOne: sessionStorage.getItem('kei_no'),
              Typename: sessionStorage.getItem('typename'),
              MenuName: NavBar.name || '',
              PageParam: JSON.stringify({Template: NavBar.type}),
              open_edition: _item ? (_item.open_edition || '') : '',
              menus_rolelist: window.btoa(window.encodeURIComponent(JSON.stringify(roles))),
              LText: '',
              LTexttb: ''
            }

            _param.LongParam = window.btoa(window.encodeURIComponent(JSON.stringify(NavBar)))
            _param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
            _param.secretkey = Utils.encrypt('', _param.timestamp)
            
            Api.getCloudConfig(_param).then(res => {
              if (res.status) {
                NavBar.open_edition = res.open_edition || ''
                window.GLOB.CacheIndependent.set(NavBar.uuid, fromJS(NavBar).toJS())

                let appViewList = sessionStorage.getItem('appViewList')
                let _appViewList = JSON.parse(appViewList)

                let index = _appViewList.findIndex(item => item.keys_id === NavBar.uuid)
                
                if (index === -1) {
                  _appViewList.unshift({
                    appkey: window.GLOB.appkey || '',
                    bid: sessionStorage.getItem('appId') || '',
                    kei_no: sessionStorage.getItem('kei_no') || '',
                    keys_id: NavBar.uuid,
                    keys_type: 'navbar',
                    remark: NavBar.name
                  })
                } else {
                  _appViewList = _appViewList.map(item => {
                    if (item.keys_id === NavBar.uuid && item.remark !== NavBar.name) {
                      item.remark = NavBar.name
                    }
                    return item
                  })
                }

                let viewList = JSON.stringify(_appViewList)

                if (appViewList !== viewList) {
                  let kparam = {
                    func: 's_kei_link_keyids_addupt',
                    BID: sessionStorage.getItem('appId'),
                    exec_type: 'x',
                    LText: ''
                  }
        
                  kparam.LText = _appViewList.map(item => `select '${item.keys_id}','${item.keys_type}','${item.kei_no}','${item.appkey}','${item.bid}','${sessionStorage.getItem('CloudUserID')}','${item.remark}'`)
                  kparam.LText = kparam.LText.join(' union all ')
                  kparam.LText = Utils.formatOptions(kparam.LText, 'x')
            
                  kparam.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
                  kparam.secretkey = Utils.encrypt('', kparam.timestamp)
        
                  Api.getCloudConfig(kparam).then(result => {
                    if (result.status) {
                      sessionStorage.setItem('appViewList', viewList)
                    }

                    resolve(result)
                  }, this.netError)
                } else {
                  resolve(res)
                }
              } else {
                resolve(res)
              }
            }, this.netError)
          }
        }
      }).then(res => { // 页面保存
        if (!res || !res.status) return res

        return Api.getCloudConfig(param)
      }).then(res => {
        this.setState({
          menuloading: false
        })

        if (!res) return

        if (res.status) {
          config.open_edition = res.open_edition || ''

          this.setState({
            config,
            oriConfig: fromJS(config).toJS(),
            needUpdate: false
          })

          localStorage.setItem('mobUpdate', new Date().getTime() + ',' + config.uuid)

          notification.success({
            top: 92,
            message: '保存成功',
            duration: 2
          })
        } else {
          Modal.warning({
            width: 400,
            title: res.message,
            okText: '知道了'
          })
        }
        MKEmitter.emit('completeSave')
      }, this.netError)
    }, 300 + (+sessionStorage.getItem('mkDelay')))
  }

  netError = (error) => {
    this.setState({
      menuloading: false
    })

    if (!error) {
      notification.warning({
        top: 92,
        message: '保存失败，请检查网络是否正常。',
        duration: 5
      })
    }
    MKEmitter.emit('completeSave')
  }

  getRoleFields = () => {
    if (sessionStorage.getItem('sysRoles')) return

    Api.getCloudConfig({func: 'sPC_Get_Roles_sModular'}).then(res => {
      if (res.status) {
        let _permFuncField = []
        let _sysRoles = []

        if (res.Roles && res.Roles.length > 0) {
          _sysRoles = res.Roles.map(role => {
            return {
              uuid: Utils.getuuid(),
              value: role.RoleID,
              text: role.RoleName
            }
          })
        }

        if (res.sModular && res.sModular.length > 0) {
          res.sModular.forEach(field => {
            if (field.ModularNo) {
              _permFuncField.push(field.ModularNo)
            }
          })
          _permFuncField = _permFuncField.sort()
        }

        sessionStorage.setItem('sysRoles', JSON.stringify(_sysRoles))
        sessionStorage.setItem('permFuncField', JSON.stringify(_permFuncField))
      }
    })
  }

  onEnabledChange = () => {
    const { config } = this.state

    let _config = {...config, enabled: !config.enabled}

    delete _config.force

    if (!_config.enabled) {
      this.setState({
        config: _config
      })
    } else if (this.verifyConfig(_config)) {
      this.setState({
        config: _config
      })
    }
  }

  verifyConfig = (config) => {
    let error = ''
    let searchSum = 0
    let swipes = []

    let check = (components, level, sign) => {
      components.forEach(item => {
        if (error) return
        if (item.type === 'tabs') {
          item.subtabs.forEach(tab => {
            check(tab.components, level + 1, tab.uuid)
          })
          return
        } else if (item.type === 'group') {
          check(item.components, level, sign)
          return
        } else if (item.type === 'navbar' && !item.wrap.MenuNo) {
          error = `导航栏《${item.name}》未设置菜单参数！`
        } else if (item.type === 'topbar') {
          if (item.wrap.type === 'search' || item.wrap.type === 'searchIcon' || (item.wrap.type === 'navbar' && item.wrap.search === 'true')) {
            searchSum += 1
          }
        } else if (item.type === 'search') {
          searchSum += 1
          if (!item.wrap.field) {
            error = `搜索条件《${item.name}》未设置搜索字段！`
          }
        }
        if (item.wrap && item.wrap.pagestyle === 'slide' && item.pageable && item.setting.laypage !== 'false') {
          // swipes.push(item.name)
          swipes.push({level, name: item.name, sign})
        }

        item.errors && item.errors.forEach(err => {
          if (err.level !== 0 || error) return
          error = `组件《${item.name}》${err.detail}`
        })
      })
    }

    check(config.components, 1, 'view')

    if (!error && searchSum > 1) {
      error = '搜索组件与导航栏的搜索功能不可同时使用。'
    } else if (!error && swipes.length > 0) {
      swipes.forEach(cell => {
        if (!error && cell.level > 2) {
          error = `不可在多层标签页中使用滑动加载组件。（${cell.name}）`
        }
      })
      if (!error) {
        let levels = swipes.map(s => s.level)
        levels = Array.from(new Set(levels))
        if (levels.length > 1) {
          error = `不可在页面与标签页中同时使用滑动加载组件。（${swipes.map(s => s.name).join('、')}）`
        }
      }

      if (!error) {
        let signs = swipes.map(s => s.sign)
        signs = Array.from(new Set(signs))
        if (signs.length !== swipes.length) {
          error = `页面中（或同一标签页中）不可同时使用多个滑动加载组件。（${swipes.map(s => s.name).join('、')}）`
        }
      }
    }

    if (config.enabled && error) {
      notification.warning({
        top: 92,
        message: error,
        duration: 5
      })
    }

    return error === ''
  }

  // 更新配置信息
  updateConfig = (config) => {
    this.setState({
      config: config
    })

    window.GLOB.customMenu = config
  }

  insert = (item) => {
    let config = fromJS(this.state.config).toJS()

    if (item.type === 'topbar' && !config.style.paddingTop) {
      config.style.paddingTop = '50px'
    }

    config.components.push(item)

    this.setState({config})
    window.GLOB.customMenu = config
  }

  setHomeView = () => {
    const { oriConfig, config } = this.state

    if (!is(fromJS(oriConfig || {}), fromJS(config || {}))) {
      notification.warning({
        top: 92,
        message: '配置信息未保存！',
        duration: 5
      })
      return
    }

    let param = {
      func: 's_kei_link_keyids_addupt',
      BID: sessionStorage.getItem('appId'),
      exec_type: 'x',
      LText: ''
    }

    let appViewList = sessionStorage.getItem('appViewList')
    appViewList = appViewList ? JSON.parse(appViewList) : []
    appViewList = appViewList.filter(item => item.keys_type !== 'index')

    appViewList.unshift({
      appkey: window.GLOB.appkey || '',
      bid: sessionStorage.getItem('appId') || '',
      kei_no: sessionStorage.getItem('kei_no') || '',
      keys_id: config.MenuID,
      keys_type: 'index',
      remark: config.MenuName
    })

    param.LText = appViewList.map(item => `select '${item.keys_id}','${item.keys_type}','${item.kei_no}','${item.appkey}','${item.bid}','${sessionStorage.getItem('CloudUserID')}','${item.remark}'`)
    param.LText = param.LText.join(' union all ')
    param.LText = Utils.formatOptions(param.LText, 'x')

    param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
    param.secretkey = Utils.encrypt('', param.timestamp)

    confirm({
      title: '确定设置本页面为首页吗？',
      content: '',
      onOk() {
        Api.getCloudConfig(param).then(result => {
          if (!result.status) {
            notification.warning({
              top: 92,
              message: result.message,
              duration: 5
            })
          } else {
            sessionStorage.setItem('appViewList', JSON.stringify(appViewList))
          }
        })
      },
      onCancel() {}
    })
  }

  setLoginView = () => {
    const { oriConfig, config } = this.state

    if (!is(fromJS(oriConfig || {}), fromJS(config || {}))) {
      notification.warning({
        top: 92,
        message: '配置信息未保存！',
        duration: 5
      })
      return
    }

    let param = {
      func: 's_kei_link_keyids_addupt',
      BID: sessionStorage.getItem('appId'),
      exec_type: 'x',
      LText: ''
    }

    let appViewList = sessionStorage.getItem('appViewList')
    appViewList = appViewList ? JSON.parse(appViewList) : []
    appViewList = appViewList.filter(item => item.keys_type !== 'login')

    appViewList.unshift({
      appkey: window.GLOB.appkey || '',
      bid: sessionStorage.getItem('appId') || '',
      kei_no: sessionStorage.getItem('kei_no') || '',
      keys_id: config.MenuID,
      keys_type: 'login',
      remark: config.MenuName
    })

    param.LText = appViewList.map(item => `select '${item.keys_id}','${item.keys_type}','${item.kei_no}','${item.appkey}','${item.bid}','${sessionStorage.getItem('CloudUserID')}','${item.remark}'`)
    param.LText = param.LText.join(' union all ')
    param.LText = Utils.formatOptions(param.LText, 'x')

    param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss')
    param.secretkey = Utils.encrypt('', param.timestamp)

    let hasLogin = config.components.findIndex(item => item.type === 'login') > -1

    confirm({
      title: '确定设置本页面为登录页吗？',
      content: hasLogin ? '' : '当前页面尚未添加登录组件。',
      onOk() {
        Api.getCloudConfig(param).then(result => {
          if (!result.status) {
            notification.warning({
              top: 92,
              message: result.message,
              duration: 5
            })
          } else {
            sessionStorage.setItem('appViewList', JSON.stringify(appViewList))
          }
        })
      },
      onCancel() {}
    })
  }

  changeView = (val) => {
    if (sessionStorage.getItem('typename') !== 'pad') return
    
    if (val !== 'vertical') {
      window.GLOB.winWidth = 992
      window.GLOB.winHeight = 690
      window.GLOB.shellWidth = 853
      window.GLOB.shellHeight = 640
    } else {
      window.GLOB.winWidth = 736
      window.GLOB.winHeight = 945
      window.GLOB.shellWidth = 640
      window.GLOB.shellHeight = 853
    }

    this.setState({
      comloading: true
    }, () => {
      this.setState({ comloading: false })
    })
  }

  resetConfig = (config) => {
    this.setState({
      config: config,
      comloading: true
    }, () => {
      this.setState({
        comloading: false
      })
    })

    window.GLOB.customMenu = config
  }

  checklog = () => {
    const { oriConfig, config } = this.state

    return is(fromJS(oriConfig), fromJS(config))
  }

  updateLogConfig = (config) => {
    config.open_edition = this.state.config.open_edition || ''
    
    this.setState({
      config: null
    }, () => {
      this.setState({
        config: config
      })
    })

    window.GLOB.customMenu = config
  }


  render () {
    const { view, viewType, comloading, loading, settingshow, controlshow, activeKey, MenuId, config, menuloading, adapters, eyeopen, needUpdate } = this.state

    return (
      <ConfigProvider locale={antdZhCN}>
        <div className={'mk-mob-view ' + viewType} id="mk-mob-design-view">
          <Header/>
          {loading ? <Spin className="view-spin" size="large" /> : null}
          <DndProvider backend={HTML5Backend}>
            {view !== 'popview' ? <><div className={'menu-setting ' + (!settingshow ? 'hidden' : '')}>
              <div className="draw">
                {settingshow ? <DoubleLeftOutlined onClick={() => {this.setState({settingshow: false})}} /> : null}
                {!settingshow ? <DoubleRightOutlined onClick={() => {this.setState({settingshow: true})}} /> : null}
              </div>
              <div className="pc-setting-tools">
                <Collapse accordion activeKey={activeKey} bordered={false} onChange={(key) => this.setState({activeKey: key})}>
                  {/* 基本信息 */}
                  <Panel header="基本信息" forceRender className="basedata" key="basedata">
                    {config ? <>
                      <MenuForm
                        config={config}
                        MenuId={MenuId}
                        adapters={adapters}
                        updateConfig={(con) => {
                          this.updateConfig(con)
                          if (con.direction !== config.direction) {
                            this.changeView(con.direction)
                          }
                        }}
                      />
                      <UrlFieldComponent config={config} updateConfig={this.updateConfig}/>
                      {/* 表名添加 */}
                      <TableComponent config={config} updatetable={this.updateConfig}/>
                      <Paragraph style={{padding: '15px 0px 0px 18px'}} copyable={{ text: MenuId }}>菜单ID</Paragraph>
                      <Paragraph style={{padding: '10px 0px 0px 18px'}} copyable={{ text:  `${window.GLOB.baseurl}mob/index.html#/index/${sessionStorage.getItem('kei_no')}/${sessionStorage.getItem('typename')}/${sessionStorage.getItem('lang')}/${MenuId}/@BID@` }}>菜单链接</Paragraph>
                      <NormalCss config={config} updateConfig={this.updateConfig}/>
                    </> : null}
                  </Panel>
                  {/* 组件添加 */}
                  <Panel header="组件" className="component" key="component">
                    <SourceWrap />
                  </Panel>
                  <Panel header="元素" key="element">
                    <Modulecell />
                  </Panel>
                  <Panel header="页面样式" key="background">
                    {config ? <BgController config={config} updateConfig={this.updateConfig} /> : null}
                  </Panel>
                </Collapse>
              </div>
            </div>
            <div className={'menu-control' + (!controlshow ? ' hidden' : '')}>
              <div className="draw">
                {controlshow ? <DoubleRightOutlined onClick={() => {this.setState({controlshow: false})}}/> : null}
                {!controlshow ? <DoubleLeftOutlined onClick={() => {this.setState({controlshow: true})}}/> : null}
              </div>
              <div className="wrap">
                <Button type="primary" className={needUpdate ? 'update-tip' : ''} onClick={this.submitConfig} id="save-config" loading={menuloading}>保存</Button>
                {config ? <Switch className="big" checkedChildren="启" unCheckedChildren="停" checked={config.enabled} onChange={this.onEnabledChange} /> : null}
                <ArrowLeftOutlined title="后退" className="back-view" onClick={this.backView}/>
                {config ? <Debug config={config}/> : null}
                <Button className="mk-border-purple" onClick={() => this.setState({eyeopen: !eyeopen})}>{!eyeopen ? <EyeOutlined /> : <EyeInvisibleOutlined />} 组件名</Button>
                <CreateView resetmenu={this.getAppMenus} />
                <PasteController insert={this.insert} />
                <TableNodes config={config} />
                <ViewNodes config={config} MenuId={MenuId}/>
                <SysInterface config={config} updateConfig={this.updateConfig}/>
                <PictureController/>
                <Quotecomponent config={config} updateConfig={this.updateConfig}/>
                <StyleCombControlButton menu={config} />
                <Button className="mk-border-green set-home" onClick={this.setHomeView}><HomeOutlined /> 设为首页</Button>
                <Button className="mk-border-purple set-login" onClick={this.setLoginView}><LoginOutlined /> 设为登录页</Button>
                <ReplaceField type="custom" config={config} updateConfig={this.resetConfig}/>
                <Transfer MenuID={MenuId} />
                {config ? <Versions MenuId={MenuId} Template="webPage" checklog={this.checklog} updateConfig={this.updateLogConfig}/> : null}
                <Button type="default" onClick={this.closeView}>关闭</Button>
              </div>
            </div>
            <div className={'menu-body menu-view' + (menuloading ? ' saving' : '') + (eyeopen ? ' eye-open' : '')}>
              {config && !comloading ? <div className="mob-shell" style={{width: window.GLOB.shellWidth, height: window.GLOB.shellHeight}}>
                <MobShell menu={config} handleList={this.updateConfig} />
              </div> : null}
            </div></> : <PopView btn={this.state.popConfig} save={this.submitPopConfig} cancel={this.closePop}/>}
          </DndProvider>
          <StyleController />
          <StyleCombController />
          <ModalController />
          <SearchController />
        </div>
      </ConfigProvider>
    )
  }
}

export default withRouter(MobDesign)